#include "Arduino.h"
#include "DFRobotDFPlayerMini.h"  // bibliotheque pour le DFPlayer
#include "Mp3.h"

DFRobotDFPlayerMini myDFPlayer;

HardwareSerial MySerial(1);
int volume = MP3_VOLUME_INI;          // Initial volume
int CurrentFileNumber = 0;                            // File number currently played
int CurrrentFolderNumber = 0;                         // Folder number currently played
int FilesCountInFolder[NUMBER_OF_FOLDERS+1];

void Mp3_init() {

  // Init serial communication with DFPlayer
  MySerial.begin(9600, SERIAL_8N1, PIN_MP3_RX, PIN_MP3_TX);

  myDFPlayer.setTimeOut(500); // Définit un temps de time out sur la communication série à 500 ms

  if (!myDFPlayer.begin(MySerial, false)) {  //Utilisation de  softwareSerial pour communiquer (ne marche pas)
    Serial.println(F("MP3 - Pb communication:"));
    Serial.println(F("1.SVP verifier connexion serie!"));
    Serial.println(F("2.SVP verifier SDcard !"));
    while(true);
  }
  randomSeed(analogRead(0));    // Init random generator
  Serial.println(F("MP3 - DFPlayer Mini En ligne."));

  //----Controle volume----
  myDFPlayer.volume(volume);  // ( valeur de 0 à 30 )
  Serial.println("MP3 - Volume =" + String(volume)); 
  

  // ---- indique d'utiliser le player de carte SD interne
  myDFPlayer.outputDevice(DFPLAYER_DEVICE_SD);

  // Folders and files init
  Serial.println("MP3 - Total folder counts = " + String(myDFPlayer.readFolderCounts()));
  Serial.println("MP3 - Total file counts = " + String(myDFPlayer.readFileCounts())); //Le nombre total de fichier mp3 sur la carte ( dossier inclus )
  for (int i=1; i<=NUMBER_OF_FOLDERS; i++) {
    FilesCountInFolder[i] = myDFPlayer.readFileCountsInFolder(i);
    Serial.println("MP3 - file counts in folder " + String(i) + " = " + String(FilesCountInFolder[i])); 
  }

  // TODO: initialise order of file playing within the folder in order to improve the random selection
  
  // Init device busy indicator
   pinMode(PIN_MP3_BUSY, INPUT);  

  Serial.println("MP3 - Initialisation completed");
}

void Mp3_powerDown() {
  myDFPlayer.sleep(); 
}

/* Not used
int Mp3_readStatus() {
  int state = myDFPlayer.readState();
  Serial.println("Mp3 - Status = " + String(state));
  return state;
}
*/

boolean Mp3_isBusy() {
  //
  // BUSY pin low when devise is playing
  //
  if (digitalRead(PIN_MP3_BUSY) == LOW) 
    return true;
  else
    return false;
}


void Mp3_playFile(uint8_t folderNumber, uint8_t fileNumber) {
	//
	//
	Serial.print("MP3 - Playing file " + String(fileNumber));
	Serial.println(" in folder " + String(folderNumber));
  myDFPlayer.playFolder(folderNumber, fileNumber);

}


void Mp3_setFolder(int folderNumber) {
  CurrrentFolderNumber = folderNumber;
  Serial.println("MP3 - Folder number changed to: " + String(CurrrentFolderNumber));
 
}

/* not used
void Mp3_playFolder(int folderNumber) {
  //
  // Starts playing randomly full folder XXX
  //
  Serial.println("MP3 - Playing full folder " + String(folderNumber));
  CurrrentFolderNumber = folderNumber;
  CurrentFileNumber = 0;
  Mp3_playNextFile();
}
*/

void Mp3_playNextFile() {
  //myDFPlayer.next();
  int nextFile = 0;

  if (FilesCountInFolder[CurrrentFolderNumber] > 1) {
    do {
      // Pick a file randomly, different than the current one
      nextFile = random(1, FilesCountInFolder[CurrrentFolderNumber] + 1);  
    } while (nextFile == CurrentFileNumber);
  } else nextFile = 1;
  
  myDFPlayer.playFolder(CurrrentFolderNumber, nextFile); 
  Serial.println("MP3 - Play next track: " + String(nextFile));
  CurrentFileNumber = nextFile;
}

void Mp3_pause() {
  myDFPlayer.pause();
}

void Mp3_start() {
  myDFPlayer.start();
}

void Mp3_stop() {
	//
	//
  Serial.println("MP3 - Stopped playing");
	myDFPlayer.stop();
}


void Mp3_volumeUp() {
  Serial.println("MP3 - Volume up");
  if (volume <= MP3_VOLUME_MAX) {
      volume +=1;
      myDFPlayer.volume(volume); 
  }
  Serial.println("MP3 - Volume =" + String(volume)); 
}

void Mp3_volumeDown() {
  Serial.println("MP3 - Volume down");
  if (volume > 0) {
      volume -=1;
      myDFPlayer.volume(volume); 
  }
  Serial.println("MP3 - Volume =" + String(volume)); 
}

/* not used
int Mp3_getVolume() {
  volume = myDFPlayer.readVolume();  
  Serial.println("MP3 - Volume =" + String(volume));  
}
*/
